<?php
namespace Teknisa\CTB\Services;

use Teknisa\Libs\Util\Environment;
use \Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\CTB\Factories\Ger06100ContaContabilFactory;

class Ger06100ContaContabilService {

    protected $nrorg;
    protected $operador;
    protected $currentTime;
    protected $environment;
    protected $entityManager;
    protected $newCode;
    protected $dataSourceManager;
    protected $factory;

    public function __construct(Environment $environment, EntityManager $entityManager, NewCode $newCode, $dataSourceManager, Ger06100ContaContabilFactory $factory) {
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->newCode = $newCode;
        $this->dataSourceManager = $dataSourceManager;
        $this->factory = $factory;
        $this->nrorg       = $this->environment->getNrOrg();
        $this->operador    = $this->environment->getCdOperador();
        $this->currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
    }

    public function geraNovoCodigo($contador, $nrorg){
        try {
            return $this->newCode->getCodeByTableName($contador, $nrorg);
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function retornaMaiorConta($row){
        try {
           return $this->factory->retornaMaiorConta($row['PAI'],$this->nrorg);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function verificaExclusao($row){
        try {
           return $this->factory->verificaExclusao($row['CDNVCONTCTBL'], $row['CDCONTCTBL']);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function salvarContaContabil($row){
        try {
            if($row['__is_new']){
                $this->validaChaveContaContabil($row);
                $this->validaContaNivelAnterior($row);
            }
            $this->validaCodigoReduzido($row);
            $this->setCamposAuditoria($row);
            $row['DTULTATUCCTB'] = $this->currentTime;
            $dataSet = new DataSet("contctbl", [$row]);
            $this->dataSourceManager->persist($dataSet);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function deletarSubContas($row){
        try {
            $dataSet = new DataSet("contctblcorre_grupocontacorre_natucontcorre", [$row]);
            $this->dataSourceManager->delete($dataSet);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function salvarSubContas($row){
        try {
            if($row['__is_new'])
                $row ['NRSEQUENCIAL'] = $this->geraNovoCodigo('CONTACORRELAT',$this->nrorg);    
            $this->validaSubContas($row);
            $this->setCamposAuditoria($row);
            $dataSet = new DataSet("contctblcorre_grupocontacorre_natucontcorre", [$row]);
            $this->dataSourceManager->persist($dataSet);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function setCamposAuditoria ($row){
        if($row['__is_new']){
            $row['NRORG']          = $this->nrorg;
            $row['IDATIVO']        = 'S';
            $row['DTINCLUSAO']     = $this->currentTime;
            $row['DTULTATU']       = $this->currentTime;
            $row['NRORGINCLUSAO']  = $this->nrorg;
            $row['CDOPERINCLUSAO'] = $this->operador;
            $row['NRORGULTATU']    = $this->nrorg;
            $row['CDOPERULTATU']   = $this->operador;
        }else{
            $row['DTULTATU']       = $this->currentTime;
            $row['NRORGULTATU']    = $this->nrorg;
            $row['CDOPERULTATU']   = $this->operador;    
        }
    }
    
    public function validaSubContas ($row){
        $resul = $this->factory->validaSubContas($row['CDCONTCTBL'], $row['NRSEQGRUPO'], $row['CDCONTACORRE'], $row['NRSEQNATU']);
        if( count($resul) >0 ){
            if( floatval ($row['NRSEQUENCIAL'] ) !== $resul[0]['NRSEQUENCIAL'] ){
                throw new \Exception('Conta correlata já cadastrada com as parametrizações informadas.', 1);
            }
        }
    }
    
    public function validaChaveContaContabil ($row){
        $resul = $this->factory->verificaContaContabil($row['CDCONTCTBL'], $this->nrorg);
        if( count($resul) >0 ){
            throw new \Exception('Conta Contábil já cadastrada.', 1);
            
        }   
    }
    
    public function validaCodigoReduzido ($row){
        $resul = $this->factory->validaCodigoReduzido($row['CDCONTREDU'], $this->nrorg);
        if( count($resul) >0 ){
            if( $resul[0]['CDCONTCTBL']!== $row['CDCONTCTBL'])
                throw new \Exception('Código Reduzido já cadastrado.', 1);
            
        }   
    }

    public function salvarCadastramentoAutomatico($row){
        try {
            switch ($row['TIPOCADASTRO']) {
                //cadastramento automático por 'Identificador Padrão Relatório'.
                case '1':
                    $this->factory->cadastroAutoIdentificadorPadrao( $row['P_IDPADRREL'], $row['P_CONTAINICIAL'], $row['P_CONTAFINAL'], $this->nrorg );
                    break;
                //cadastramento automático por 'Conta Contábil Base (Análise Vertical)'.
                case '2':
                    $this->factory->cadastroAutoContaVertical( $row['P_CONTABASE'], $row['P_CONTAINICIAL'], $row['P_CONTAFINAL'], $this->nrorg );
                    break;
                //cadastramento automático por 'Alterar Conta Referencial'.
                case '3':
                    if( $row['TIPOCONTA'] === 'ECD' )
                        $this->factory->cadastroAutoContaReferencialEcd( $row['P_CONTANTIGA'], $row['P_CONTNOVA'], $this->nrorg );
                    else if( $row['TIPOCONTA'] === 'AUX' )
                        $this->factory->cadastroAutoContaReferencialAux( $row['P_CONTANTIGA'], $row['P_CONTNOVA'], $this->nrorg );
                    break;
            }
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function validaContaNivelAnterior ($row){
        //contas de nivel 1 não possuem PAI
        if( floatval($row['CDNVCONTCTBL']) > 1 ){
            $resul = $this->factory->verificaContaContabil($row['CDCONTANTE'], $this->nrorg);
            if( count($resul) === 0 )
                throw new \Exception('Nível anterior da conta contábil não cadastrado.', 1);
        }
    }
    
}